@@ -17,7 +17,8 @@
 #' @param modeValue logical indicates if mode values should be computed. Default is FALSE.
 #' @param percentiles list of percentiles (integers between 0 and 100) to collect (always collects 25th and 75th 
 #'   for IQR calculation). There is no penalty in specifying more percentiles as they get calculated in a single call 
-#'   for each column - no matter how many different values are requested.
+#'   for each column - no matter how many different values are requested. When FALSE then percentiles 
+#'   calculations are skipped and result wil have no percentile and IQR columns.
 #' @param parallel logical: enable parallel calls to Aster database. This option requires parallel 
 #'   backend enabled and registered (see in examples). Parallel execution requires ODBC \code{channel} 
 #'   obtained without explicit password: either with \code{\link{odbcConnect}(dsn)} or 
@@ -102,33 +103,58 @@ getTableSummary <- function (channel, tableName, include = NULL, except = NULL,
   
   where_clause = makeWhereClause(where)
   
+  # derive if percentiles are numeric or logical
+  if (is.numeric(percentiles)) {
+    percentileFlag = TRUE
+  }else if(is.logical(percentiles) && length(percentiles) == 1){
+    if(percentiles) {
+      percentileFlag = TRUE
+      percentiles = NULL
+    }else
+      percentileFlag = FALSE
+  }else
+    stop("Percentiles must be either a numeric vector or a logical flag.")
+  
   # percentiles
-  # always add 50 (median) and 25, 75 for IQR computations
-  if (is.null(percentiles) || !is.numeric(percentiles)) {
-    percentiles = c(25, 50, 75)
-  }else {
-    percentiles = union(percentiles, c(25, 50, 75)) 
-  }
-  out_of_range = percentiles < 0 | percentiles > 100
-  if (any(out_of_range)) {
-    stop(paste("Invalid percentile value(s) passed (below 0 or above 100):", percentiles[out_of_range]))
-  }
+  if (percentileFlag) {
+    # always add 50 (median) and 25, 75 for IQR computations
+    if (is.null(percentiles) || !is.numeric(percentiles)) {
+      percentiles = c(25, 50, 75)
+    }else {
+      percentiles = union(percentiles, c(25, 50, 75)) 
+    }
+    out_of_range = percentiles < 0 | percentiles > 100
+    if (any(out_of_range)) {
+      stop(paste("Invalid percentile value(s) passed (below 0 or above 100):", percentiles[out_of_range]))
+    }
+  }else
+    percentiles = NULL
   
   # no database access if mock is TRUE
   if (mock) {
     return (table_info)
   }
   
-  percentileNames = paste0(percentiles, "%")
-  percentileStrNames = paste0(percentiles, "%str")
-  names(percentileNames) = percentiles
-  names(percentileStrNames) = percentiles
-  percentileStr = paste(percentiles, collapse=",")
+  if (percentileFlag) {
+    percentileNames = paste0(percentiles, "%")
+    percentileStrNames = paste0(percentiles, "%str")
+    names(percentileNames) = percentiles
+    names(percentileStrNames) = percentiles
+    percentileStr = paste(percentiles, collapse=",")
+  }else {
+    percentileNames = NULL
+    percentileStrNames= NULL
+    percentileStr = NULL
+  }
   
   # Create data frame columns for table statistics
   table_info[c("total_count","distinct_count","not_null_count","null_count")] = as.integer(NA)
-  table_info[c("minimum","maximum","average","deviation", percentileNames, "IQR")] = as.numeric(NA)
-  table_info[c("minimum_str","maximum_str", "average_str", percentileStrNames)] = as.character(NA)
+  table_info[c("minimum","maximum","average","deviation")] = as.numeric(NA)
+  if (percentileFlag) 
+    table_info[c(percentileNames, "IQR")] = as.numeric(NA)
+  table_info[c("minimum_str","maximum_str", "average_str")] = as.character(NA)
+  if (percentileFlag)
+    table_info[c(percentileStrNames)] = as.character(NA)
   
   # Total rows
   total_rows = toaSqlQuery(channel, paste0("SELECT COUNT(*) cnt FROM ", tableName, where_clause))
@@ -143,14 +169,17 @@ getTableSummary <- function (channel, tableName, include = NULL, except = NULL,
   
   if (length(numeric_columns) > 0) {
     metrics = computeNumericMetrics(channel, tableName, table_info, numeric_columns, 
-                                    percentileNames, percentileStr, 
+                                    percentileFlag, percentileNames, percentileStr, 
                                     where_clause, total_count, parallel)
-    
+    if (percentileFlag)
+      percentile_names = c(percentileNames, "IQR")
+    else
+      percentile_names = NULL
     table_info[metrics$idx, 
                c("distinct_count", "not_null_count", "null_count", 
-                 "minimum", "maximum", "average", "deviation", percentileNames, "IQR")] =
+                 "minimum", "maximum", "average", "deviation", percentile_names)] =
       metrics[, c("distinct_count", "not_null_count", "null_count", 
-                  "minimum", "maximum", "average", "deviation", percentileNames, "IQR")]
+                  "minimum", "maximum", "average", "deviation", percentile_names)]
   }
   
   
@@ -161,8 +190,9 @@ getTableSummary <- function (channel, tableName, include = NULL, except = NULL,
     metrics = computeCharacterMetrics(channel, tableName, table_info, character_columns, 
                                        where_clause, total_count, parallel)
     
-    table_info[metrics$idx, c("distinct_count", "not_null_count", "null_count", 
-                              "minimum_str", "maximum_str")] =
+    table_info[metrics$idx, 
+               c("distinct_count", "not_null_count", "null_count",
+                 "minimum_str", "maximum_str")] =
       metrics[, c("distinct_count", "not_null_count", "null_count", 
                   "minimum_str", "maximum_str")]
   }
@@ -172,15 +202,20 @@ getTableSummary <- function (channel, tableName, include = NULL, except = NULL,
   
   if (length(temporal_columns) > 0) {
     metrics = computeTemporalMetrics(channel, tableName, table_info, temporal_columns,
-                                     percentiles, percentileNames, percentileStrNames, percentileStr, 
+                                     percentileFlag, percentiles, percentileNames, percentileStrNames, percentileStr, 
                                      where_clause, total_count, parallel)
     
-    table_info[metrics$idx, c("distinct_count", "not_null_count", "null_count",
-                              "minimum_str", "maximum_str", "average_str", 
-                              percentileNames, "IQR", percentileStrNames)] =
+    if (percentileFlag)
+      percentile_names = c(percentileNames, "IQR", percentileStrNames)
+    else
+      percentile_names = NULL
+    table_info[metrics$idx, 
+               c("distinct_count", "not_null_count", "null_count",
+                 "minimum_str", "maximum_str", "average_str",
+                 percentile_names)] =
       metrics[, c("distinct_count", "not_null_count", "null_count",
                   "minimum_str", "maximum_str", "average_str", 
-                  percentileNames, "IQR", percentileStrNames)]
+                  percentile_names)]
   }
   
   # Compute modes
@@ -200,7 +235,7 @@ getTableSummary <- function (channel, tableName, include = NULL, except = NULL,
   
 }
 
-makeNumericMetrics <- function(idx, column_stats, total_count, presults, percentileNames) {
+makeNumericMetrics <- function(idx, column_stats, total_count, percentileFlag, presults, percentileNames) {
   df = data.frame(
              idx=idx,
              distinct_count=column_stats[[1,"distinct_count"]],
@@ -212,27 +247,29 @@ makeNumericMetrics <- function(idx, column_stats, total_count, presults, percent
              deviation=column_stats[[1,"deviation"]],
              stringsAsFactors=FALSE)
   
-  if (nrow(presults) > 0) {
-    for (tile in 1:nrow(presults)) {
-      ptile = as.character(presults[[tile, "percentile"]])
-      ptileValue = presults[[tile, "value"]]
-      df[1, percentileNames[ptile]] = ptileValue
+  if (percentileFlag) {
+    if (nrow(presults) > 0) {
+      for (tile in 1:nrow(presults)) {
+        ptile = as.character(presults[[tile, "percentile"]])
+        ptileValue = presults[[tile, "value"]]
+        df[1, percentileNames[ptile]] = ptileValue
+      }
+      
+      df[1, "IQR"] = presults[[which(presults$percentile==75),"value"]] -
+        presults[[which(presults$percentile==25),"value"]]
+    }else {
+      for (ptileName in percentileNames)
+        df[1, ptileName] = NA
+      df[1, "IQR"] = NA
     }
-    
-    df[1, "IQR"] = presults[[which(presults$percentile==75),"value"]] -
-      presults[[which(presults$percentile==25),"value"]]
-  }else {
-    for (ptileName in percentileNames)
-      df[1, ptileName] = NA
-    df[1, "IQR"] = NA
   }
   
   return (df)
 }
 
 computeNumericMetrics <- function(channel, tableName, tableInfo, numeric_columns,
-                                  percentileNames, percentileStr, where_clause, 
-                                  total_count, parallel=FALSE) {
+                                  percentileFlag, percentileNames, percentileStr, 
+                                  where_clause, total_count, parallel=FALSE) {
   
   metricsSql = 
     paste0("SELECT cast(count(distinct (%%%column__name%%%)) as bigint) as distinct_count, ",
@@ -269,10 +306,13 @@ computeNumericMetrics <- function(channel, tableName, tableInfo, numeric_columns
                                                           fixed=TRUE), stringsAsFactors = FALSE)
                          
                  # compute all percentiles at once with SQL/MR approximate percentile function
-                 presults = toaSqlQuery(channel, gsub('(%%%column__name%%%)', column_name, percentileSql, 
-                                                      fixed=TRUE), stringsAsFactors = FALSE)
+                 if (percentileFlag)
+                   presults = toaSqlQuery(channel, gsub('(%%%column__name%%%)', column_name, percentileSql, 
+                                                        fixed=TRUE), stringsAsFactors = FALSE)
+                 else
+                   presults = NULL
                          
-                 makeNumericMetrics(idx, column_stats, total_count, presults, percentileNames)
+                 makeNumericMetrics(idx, column_stats, total_count, percentileFlag, presults, percentileNames)
                }  
             }
   }else {
@@ -289,12 +329,16 @@ computeNumericMetrics <- function(channel, tableName, tableInfo, numeric_columns
                                          stringsAsFactors = FALSE, closeOnError=TRUE)
                          
                  # compute all percentiles at once with SQL/MR approximate percentile function
-                 presults = toaSqlQuery(parChan, gsub('(%%%column__name%%%)', column_name, percentileSql, 
-                                                      fixed=TRUE), 
-                                     stringsAsFactors = FALSE, closeOnError=TRUE)
+                 if (percentileFlag)
+                   presults = toaSqlQuery(parChan, gsub('(%%%column__name%%%)', column_name, percentileSql, 
+                                                        fixed=TRUE), stringsAsFactors = FALSE, 
+                                          closeOnError=TRUE)
+                 else
+                   presults = NULL
+                 
                  close(parChan)
                          
-                 makeNumericMetrics(idx, column_stats, total_count, presults, percentileNames)
+                 makeNumericMetrics(idx, column_stats, total_count, percentileFlag, presults, percentileNames)
               }  
             }
   }
@@ -356,7 +400,7 @@ computeCharacterMetrics <- function(channel, tableName, tableInfo, character_col
   return (result)
 }
 
-makeTemporalMetrics <- function(idx, column_stats, total_count, presults, percentileNames, percentileStrNames) {
+makeTemporalMetrics <- function(idx, column_stats, total_count, percentileFlag, presults, percentileNames, percentileStrNames) {
   df = data.frame(
     idx=idx,
     distinct_count=column_stats[[1,"distinct_count"]],
@@ -367,33 +411,35 @@ makeTemporalMetrics <- function(idx, column_stats, total_count, presults, percen
     average_str=column_stats[[1,"average"]],
     stringsAsFactors=FALSE)
   
-  if (nrow(presults) > 0) {
-    for (tile in 1:nrow(presults)) {
-      ptile = as.character(presults[[tile, "percentile"]])
-      ptileValue = presults[[tile, "epoch"]]
-      df[1, percentileNames[ptile]] = ptileValue
+  if (percentileFlag) {
+    if (nrow(presults) > 0) {
+      for (tile in 1:nrow(presults)) {
+        ptile = as.character(presults[[tile, "percentile"]])
+        ptileValue = presults[[tile, "epoch"]]
+        df[1, percentileNames[ptile]] = ptileValue
+        
+        ptileValue = presults[[tile, "value"]]
+        df[1, percentileStrNames[ptile]] = ptileValue
+      }
       
-      ptileValue = presults[[tile, "value"]]
-      df[1, percentileStrNames[ptile]] = ptileValue
+      df[1, "IQR"] = presults[[which(presults$percentile==75),"epoch"]] -
+        presults[[which(presults$percentile==25),"epoch"]]
+    }else {
+      for (ptileName in percentileNames)
+        df[1, ptileName] = NA
+      
+      for (ptileStrName in percentileStrNames)
+        df[1, ptileStrName] = NA
+      
+      df[1, "IQR"] = NA
     }
-    
-    df[1, "IQR"] = presults[[which(presults$percentile==75),"epoch"]] -
-      presults[[which(presults$percentile==25),"epoch"]]
-  }else {
-    for (ptileName in percentileNames)
-      df[1, ptileName] = NA
-    
-    for (ptileStrName in percentileStrNames)
-      df[1, ptileStrName] = NA
-    
-    df[1, "IQR"] = NA
   }
   
   return (df)
 }
 
 computeTemporalMetrics <- function(channel, tableName, tableInfo, temporal_columns,
-                                   percentiles, percentileNames, percentileStrNames, percentileStr, 
+                                   percentileFlag, percentiles, percentileNames, percentileStrNames, percentileStr, 
                                    where_clause, total_count, parallel=FALSE) {
   
   where_clause = ifelse(length(where_clause)>1, paste0(where_clause, " AND (%%%column__name%%%) IS NOT NULL"),
@@ -448,8 +494,11 @@ computeTemporalMetrics <- function(channel, tableName, tableInfo, temporal_colum
                                                     stringsAsFactors = FALSE)
                          
                          # compute all percentiles at once with SQL/MR approximate percentile function
-                         presults = toaSqlQuery(channel, gsub('(%%%column__name%%%)', column_name, percentileSql, 
+                         if (percentileFlag)
+                           presults = toaSqlQuery(channel, gsub('(%%%column__name%%%)', column_name, percentileSql, 
                                                               fixed=TRUE), stringsAsFactors = FALSE)
+                         else
+                           presults = NULL
                          
                          makeTemporalMetrics(idx, column_stats, total_count, presults, percentileNames, percentileStrNames)
                        }
@@ -469,9 +518,13 @@ computeTemporalMetrics <- function(channel, tableName, tableInfo, temporal_colum
                                                     stringsAsFactors = FALSE, closeOnError=TRUE)
                          
                          # compute all percentiles at once with SQL/MR approximate percentile function
-                         presults = toaSqlQuery(parChan, gsub('(%%%column__name%%%)', column_name, percentileSql, 
-                                                              fixed=TRUE), 
-                                                stringsAsFactors = FALSE, closeOnError=TRUE)
+                         if (percentileFlag)
+                           presults = toaSqlQuery(parChan, gsub('(%%%column__name%%%)', column_name, percentileSql, 
+                                                                fixed=TRUE), stringsAsFactors = FALSE, 
+                                                  closeOnError=TRUE)
+                         else
+                           presults = NULL
+                         
                          close(parChan)
                          
                          makeTemporalMetrics(idx, column_stats, total_count, presults, percentileNames, percentileStrNames)
@@ -615,11 +668,24 @@ getColumnValues <- function(conn, tableName, columnName, where = NULL, mock = FA
 
 #' Test if table present in the database.
 #' 
+#' Tests each element if a table or a view exists in one of database schemas.
+#' Names could be just table name of schema followed by table name via '.'.
+#';
+#' 
 #' @param channel object as returned by \code{\link{odbcConnect}}.
-#' @param names vector of table names. Name may contain SQL wildcard 
-#'   characters but it should not contain schema prefix. All visible schemas 
-#'   will be checked for table name specified.
+#' @param tables vector of table names. Name may contain schema name
+#'   followed by dot and table name. All visible schemas are checked 
+#'   when table specified without a schema.
+#' @param withNames logical indicates if table components included in the
+#'   results.
+#' @param allTables a data frame containing table-like objects accessible from
+#'   Aster. Usually obtained with \code{\link{sqlTables}}.
 #' @return logical vector indicating if corresponding name is table in Aster database.
+#'   Special case when name contains a SQL query triggers \code{NA} value instead.
+#'   This behavior is handy when checking if a table name corresponds to a table 
+#'   (or a view) or a SQL query instead. For example, to test if all names are
+#'   either existing tables or views or SQL queries use something like this:
+#'   \code{result = isTable(...); if(all(result | is.na(result)) ...}
 #' @seealso \code{\link{getTableSummary}}
 #' @export
 #' @examples 
@@ -632,10 +698,52 @@ getColumnValues <- function(conn, tableName, columnName, where = NULL, mock = FA
 #' isTable(conn, "pitch%")          # TRUE
 #' isTable(conn, "public.pitching") # FALSE
 #' }
-isTable <- function(channel, names) {
-  if (is.null(names) || length(names) < 1) return(logical(0))
+isTable <- function(channel, tables, withNames=FALSE, allTables=NULL) {
   
-  vapply(names, FUN=function(x) nrow(sqlTables(channel, tableName=x))>0, FUN.VALUE=logical(1))
+  if (is.null(tables) || length(tables) < 1) return(logical(0))
+  
+  # trim leading and trailing spaces
+  tables = gsub("^[[:blank:]]+|[[:blank:]]+$", "", tables)
+  
+  if(is.null(names(tables))) 
+     row_names = tables
+  else
+     row_names = names(tables)
+  
+  df = data.frame(candidate = !grepl("[[:blank:]]", tables),
+                  schema = substr(tables, 1, regexpr("\\.[^\\.]*$", tables)-1), 
+                  table = substr(tables, regexpr("[^\\.]*$", tables), nchar(tables)), 
+                  stringsAsFactors = FALSE)
+  
+  # all tables and views
+  if(is.null(allTables)) {
+    isValidConnection(channel, FALSE)
+    allTables = sqlTables(channel)
+  }
+  
+  result = mapply(FUN = function(name, fullname, candidate, schema, table) {
+    if (!candidate) return(NA)
+    if (nchar(schema)>0) {
+      if(nrow(allTables[allTables$TABLE_NAME == table &
+                         allTables$TABLE_SCHEM == schema &
+                         paste0(allTables$TABLE_SCHEM, '.', allTables$TABLE_NAME) == fullname, ]) > 0)
+        return(TRUE)
+      else
+        return(FALSE)
+    }else {
+      if(any(allTables$TABLE_NAME == table))
+        return(TRUE)
+      else
+        return(FALSE)
+    }
+  }, row_names, tables, df$candidate, df$schema, df$table, SIMPLIFY = TRUE)
+  
+  if(withNames) {
+    result = data.frame(isTable=result, schema=df$schema, table=df$table,
+                        row.names = row_names, stringsAsFactors = FALSE)
+  }
+  
+  return(result)
 }
 
 
@@ -698,6 +806,10 @@ grantExecuteOnFunction <- function(conn, name='%', owner='db_superuser', user) {
 #'   The creteria are expressed in the form of SQL predicates (inside \code{WHERE} clause).
 #' @param output Default output is a data frame in \code{'long'} format. Other options include 
 #'   \code{'wide'} format and \code{'matrix'}.
+#' @param percent logical: if TRUE then percent of NULL values instead of absolute count returned. To avoid
+#'   division by zero small error is introduced by incrementing by 1 total count of rows in the table.
+#' @param stringsAsFactors logical: should data frame returned have column with variables 
+#'   (table column names) as factor? Applies only when the output is in \code{long} format.
 #' @param test logical: if TRUE show what would be done, only (similar to parameter \code{test} in \link{RODBC} 
 #'   functions like \link{sqlQuery} and \link{sqlSave}).
 #' @export
@@ -713,7 +825,8 @@ grantExecuteOnFunction <- function(conn, name='%', owner='db_superuser', user) {
 #' 
 #' }
 getNullCounts <- function(channel, tableName, tableInfo=NULL, include=NULL, except=NULL, 
-                          schema=NULL, output='long', where=NULL, test=FALSE){
+                          output='long', percent=FALSE, schema=NULL, where=NULL, 
+                          stringsAsFactors=FALSE, test=FALSE){
   
   # match argument values
   output = match.arg(output, c('long', 'wide', 'matrix'))
@@ -741,7 +854,7 @@ getNullCounts <- function(channel, tableName, tableInfo=NULL, include=NULL, exce
     columnNames <- includeExcludeColumns(tableInfo, include, except)$COLUMN_NAME
     
     ## per column name, construct count nulls SQL
-    columnNameNullSQL <- sapply(columnNames, constructCountNullsSQL)
+    columnNameNullSQL <- sapply(columnNames, constructCountNullsSQL, percent)
     
     where_clause <- makeWhereClause(where)
 
@@ -762,7 +875,8 @@ getNullCounts <- function(channel, tableName, tableInfo=NULL, include=NULL, exce
       ## transpose results
       nullCountsLongMatrix <- t(nullCountsWide)
       nullCountsLong <- data.frame(variable = row.names(nullCountsLongMatrix),
-                                   nullcount = as.integer(nullCountsLongMatrix))
+                                   nullcount = as.numeric(nullCountsLongMatrix),
+                                   stringsAsFactors = stringsAsFactors)
       return (nullCountsLong)
     }else
       return (as.matrix(t(nullCountsWide)))
@@ -770,7 +884,14 @@ getNullCounts <- function(channel, tableName, tableInfo=NULL, include=NULL, exce
 }
 
 ## construct count nulls string -- used in getNullCounts
-constructCountNullsSQL <- function(variableName){
-    sqlString <- paste0('COUNT(1) - COUNT(',variableName,') AS ', variableName)
-    return(sqlString)
+constructCountNullsSQL <- function(variableName, percent){
+  
+  if(percent)
+    sqlString = paste0('(COUNT(1) - COUNT(',variableName,') + 1.0)/(COUNT(1) + 1.0)')
+  else
+    sqlString = paste0('COUNT(1) - COUNT(',variableName,')')
+  
+  sqlString = paste0(sqlString, ' AS ', variableName)
+  
+  return(sqlString)
 }
\ No newline at end of file
